#include <Screen.h>
#include <TranslationKit.h>
#include <File.h>
#include <Bitmap.h>
#include <Debug.h>
#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <Font.h>
#include <NodeInfo.h>
#include <string.h>

// from interface_defs.h
struct window_info {
	team_id		team;
    int32   	id;	  		  /* window's token */

	int32		thread;
	int32		client_token;
	int32		client_port;
	uint32		workspaces;

	int32		layer;
    uint32	  	w_type;    	  /* B_TITLED_WINDOW, etc. */
    uint32      flags;	  	  /* B_WILL_FLOAT, etc. */
	int32		window_left;
	int32		window_top;
	int32		window_right;
	int32		window_bottom;
	int32		show_hide_level;
	bool		is_mini;
	char		name[1];
};


_IMPEXP_BE window_info	*get_window_info(int32 a_token);
_IMPEXP_BE int32		*get_token_list(team_id app, int32 *count);

int main()
{
	BApplication app( "application/x-vnd.takamatsu-wingrab" ); 
	
	app_info ainfo;
 	be_roster->GetActiveAppInfo(&ainfo);
	team_id	theTeam = ainfo.team;
	int32 count = 0;
	int32 *tokens = get_token_list(theTeam, &count);

	for(int32 outCount = 0; outCount < count;outCount++)
	{
	// Get front window info	
	window_info *wInfo = get_window_info(tokens[outCount]);
	// Skip menus screenshot
	if(::strncmp(wInfo->name,"cached[",7) == 0)
	{
		free( wInfo );
		continue;
	}
	// Get font window rect
	BRect windowRect;
	windowRect.left = wInfo->window_left;
	windowRect.top = wInfo->window_top;
	windowRect.bottom = wInfo->window_bottom;
	windowRect.right = wInfo->window_right;
 
	// Get window bitmap from screen
	BScreen screen;
	BRect bitmapRect = windowRect;
	bitmapRect.OffsetTo(B_ORIGIN);
	BBitmap *bitmap = new BBitmap(bitmapRect,B_RGBA32);
	if(screen.ReadBitmap(bitmap,false,&windowRect) != B_OK)
	{
		(new BAlert("","WinGrab:Could not get bitmap from screen or window position is out of screen rect","OK"))->Go();
		free(wInfo);
		break;
	}
	// Erase tabs background 
	int32 bytesPerRow = bitmap->BytesPerRow();
	
	uint32 *data = (uint32*)bitmap->Bits();
	uint32 white = 0xffffffff;
	
	rgb_color color = ui_color(B_WINDOW_TAB_COLOR);
	uint32 tab_color = 0xff000000;
	uint32 tmp = color.red;
	tab_color = tab_color|(tmp << 16);
	tmp = color.green;
	tab_color = tab_color|(tmp << 8);
	tmp = color.blue;
	tab_color = tab_color|(tmp);
	
	// Calc tab start and end
	int32 tab_end = 0;
	int32 tab_start = 0;
	bool enter_tab = false;
	uint32 current_data;
	uint32 shadow_color = 0;
	for(int32 j = 0;j < bytesPerRow/4;j++)
	{
		current_data = data[2*bytesPerRow/4+j];
		if(!enter_tab && current_data == tab_color)
		{
			enter_tab = true;
			tab_start = j -2;
		}else if(enter_tab && current_data != tab_color){
			tab_end = j+1;
			shadow_color = data[2*bytesPerRow/4+j+1];
			break;	
		}
	}
	// Calc tab height
	int32 tab_height = 0;
	if(tab_end != 0)
	{
		for(int32 i = bytesPerRow/4;i < bytesPerRow*10;i+=bytesPerRow/4)
		{
			if(data[i+tab_end] != shadow_color)
			{	
				tab_height = i-bytesPerRow/4;
				break;
			}
		}
	}
	// Change background color to white 
	for(int32 i = 0;i < tab_height;i+=bytesPerRow/4)
	{
		for(int32 j = 0;j < bytesPerRow/4;j++)
		{
			if(j > tab_end || j < tab_start)
				data[i+j] = white;
		}
	}
	// Store window bitmap		
	BTranslatorRoster *roster = BTranslatorRoster::Default(); 
	BBitmapStream stream(bitmap); 

	char name[B_FILE_NAME_LENGTH];
	::strcpy(name,"/boot/home/screen.png");
	int32 index = 1;
	BFile *file = new BFile();
	while(file->SetTo(name,B_FAIL_IF_EXISTS|B_CREATE_FILE | B_WRITE_ONLY) != B_OK)
	{
		::sprintf(name,"/boot/home/screen%ld.png",index++);
	}
	
	stream.Seek(0,SEEK_SET);
	if(roster->Translate(&stream, NULL, NULL, file,B_PNG_FORMAT) != B_OK)
		(new BAlert("","WinGrab:Could not find translator","OK"))->Go(); 
	
	// Set mime type
	BNodeInfo ninfo(file);
	ninfo.SetType("image/x-png");
	delete file;
	free( wInfo );
    break;
	}
	// Quit 
	app.PostMessage( B_QUIT_REQUESTED ); 
    app.Run();
    free( tokens );
	return 0;
}